#include <SoftwareSerial.h>
#include <TinyGPS.h>

TinyGPS gps;
SoftwareSerial ss(3, 2);
unsigned long pattern = 0x00000000; //0000 speed, 0000 error

static void smartdelay(unsigned long ms);
static void print_float(float val, float invalid, int len, int prec);
static void print_int(unsigned long val, unsigned long invalid, int len);
static void print_date(TinyGPS &gps);
static void print_str(const char *str, int len);
long readVcc();

void setup()
{
  Serial.begin(115200);
  pinMode(13, OUTPUT);
  ss.begin(9600);
}

  float flat, flon;
  unsigned long age, date, time, chars = 0, chars2 = 0;
  unsigned short sentences = 0, sentences2 = 0, failed = 0;

void loop()
{
  gps.stats(&chars, &sentences, &failed);
  if(gps.satellites() != TinyGPS::GPS_INVALID_SATELLITES && sentences != sentences2) {
    gps.f_get_position(&flat, &flon, &age);
    Serial.print(",");
    print_date(gps);
    Serial.print(",");
    Serial.print(flat,6);
    Serial.print(",");
    Serial.print(flon,6);
    Serial.print(",");
    Serial.print(gps.f_altitude(),1);
    Serial.print(",");
    Serial.println(gps.hdop());

  } 

  

  if(chars == chars2) pattern = 0xffff;
  else if(sentences == sentences2) pattern = chars % 2;
  else if(readVcc()<4000) pattern = 0;
  else pattern = (gps.speed() << 16) + gps.satellites()-4;
  
  chars2 = chars;
  sentences2 = sentences;
  
  smartdelay(1000);
}
long readVcc() { 
  long result; // Read 1.1V reference against AVcc 
  ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
  delay(2); // Wait for Vref to settle
  ADCSRA |= _BV(ADSC); // Convert
  while (bit_is_set(ADCSRA,ADSC));
  result = ADCL;
  result |= ADCH<<8;
  result = 1126400L / result; // Back-calculate AVcc in mV
  return result;
}
static void smartdelay(unsigned long ms)
{
  unsigned long start = millis();
  do 
  {
    digitalWrite(13, (pattern >> ((millis() % 1000) * 32 / 1000)) & 0x1 );
    while (ss.available())
      gps.encode(ss.read());
  } while (millis() - start < ms);
}

static void print_float(float val, float invalid, int len, int prec)
{
  if (val == invalid)
  {
    while (len-- > 1)
      Serial.print('*');
    Serial.print(' ');
  }
  else
  {
    Serial.print(val, prec);
    int vi = abs((int)val);
    int flen = prec + (val < 0.0 ? 2 : 1); // . and -
    flen += vi >= 1000 ? 4 : vi >= 100 ? 3 : vi >= 10 ? 2 : 1;
    for (int i=flen; i<len; ++i)
      Serial.print(' ');
  }
  smartdelay(0);
}

static void print_int(unsigned long val, unsigned long invalid, int len)
{
  char sz[32];
  if (val == invalid)
    strcpy(sz, "*******");
  else
    sprintf(sz, "%ld", val);
  sz[len] = 0;
  for (int i=strlen(sz); i<len; ++i)
    sz[i] = ' ';
  if (len > 0) 
    sz[len-1] = ' ';
  Serial.print(sz);
  smartdelay(0);
}

static void print_date(TinyGPS &gps)
{
  int year;
  byte month, day, hour, minute, second, hundredths;
  unsigned long age;
  gps.crack_datetime(&year, &month, &day, &hour, &minute, &second, &hundredths, &age);
  if (age == TinyGPS::GPS_INVALID_AGE)
    Serial.print("********** ******** ");
  else
  {
    char sz[32];
    sprintf(sz, "%02d-%02d-%02d %02d:%02d:%02d",
        year, month, day, hour, minute, second);
    Serial.print(sz);
  }
  //print_int(age, TinyGPS::GPS_INVALID_AGE, 5);
  smartdelay(0);
}

static void print_str(const char *str, int len)
{
  int slen = strlen(str);
  for (int i=0; i<len; ++i)
    Serial.print(i<slen ? str[i] : ' ');
  smartdelay(0);
}